/*
	Application: 	        reglib
	Author:			pagvac (Adrian Pastor)
	Filename:		reglib.cpp
	Date:			March 21, 2005
	Description:	        CReg class definition

	Copyright 2005 Adrian Pastor
	This program is distributed under the terms of the GNU General Public License
*/

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "reglib.h"


// PRE:	 HKEY containing hive, string containing key to access,
//		 string holding the value to query, 
//		 string which will contain the data held by the queried value
//		 
// POST: FALSE if operation failed, TRUE otherwise. 
//       Reasons for faiulure include lack of permissions and not existing key/value
BOOL CReg::QueryStrVal(HKEY hHive, char lszKey[255], char lszVal2Query[255], char lszValData[255])
{
     char lszResult[255];
     HKEY hKey;
     LONG returnStatus;
     DWORD dwType=REG_SZ;
	 DWORD dwSize=255;
     
	 returnStatus = RegOpenKeyEx(hHive, lszKey, 0L, KEY_READ, &hKey);
     
	 if (returnStatus == ERROR_SUCCESS)
     {
		 returnStatus = RegQueryValueEx(hKey, lszVal2Query, NULL, &dwType,(LPBYTE)&lszResult, &dwSize);
         
		 if (returnStatus == ERROR_SUCCESS)
         {
			   strcpy(lszValData, lszResult);
			   RegCloseKey(hKey);
			   return TRUE;
         }
		 // value not queried successfully
		 else
		 {
			   RegCloseKey(hKey);
			   return FALSE;
		 }
		  
     }
	 // key not opened successfully
	 else
	 {
		 RegCloseKey(hKey);
		 return FALSE;
     }	
}


// PRE:	 HKEY containing hive, string containing key to access,
//		 string containing the name of the value to query, 
//		 DWORD (32 bit unsigned integer) which will contain the queried value
//		 
// POST: FALSE if operation failed, TRUE otherwise. 
//       Reasons for faiulure include lack of permissions and not existing key/value
BOOL CReg::QueryDwordVal(HKEY hHive, char lszKey[255], char lszVal2Query[255], DWORD *dwVal)
{
    DWORD dwResult;
    HKEY hKey;
    LONG returnStatus;
	DWORD dwType=REG_DWORD;
    DWORD dwSize=sizeof(DWORD);
    
	returnStatus = RegOpenKeyEx(hHive, lszKey, 0L, KEY_READ, &hKey);

    if (returnStatus  == ERROR_SUCCESS)
    {
        returnStatus = RegQueryValueEx(hKey, lszVal2Query, NULL, &dwType,(LPBYTE)&dwResult, &dwSize);
        if (returnStatus == ERROR_SUCCESS)
        {
            *dwVal=dwResult; 
			//printf("REG_DWORD local value is %d\n", dwResult);
			RegCloseKey(hKey);
			 
			return TRUE;
        }
		else
		{
			RegCloseKey(hKey);

			return FALSE;
		}

     }

	else
	{
		RegCloseKey(hKey);

		return FALSE;
	}
}


// PRE:	 HKEY containing hive, string containing key to create
//		 
// POST: FALSE if operation failed, TRUE otherwise. 
//       Reasons for faiulure include lack of permissions
BOOL CReg::CreateKey(HKEY hHive, char lszKey[255])
{
	HKEY hKey;
    DWORD dwDisp = 0;
    LPDWORD lpdwDisp = &dwDisp;

    LONG iSuccess = RegCreateKeyEx( hHive, lszKey, 0L,NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey,lpdwDisp);

    if(iSuccess == ERROR_SUCCESS)
    {
		RegCloseKey(hKey);
		return TRUE;
    }

	else
	{
		RegCloseKey(hKey);
		return FALSE;
	}
}


// PRE:	 HKEY containing hive, string containing key to access,
//		 string containing name of value to write,
//		 string containing data to write as value
//		 
// POST: FALSE if operation failed, TRUE otherwise. 
//       Reasons for failure include lack of permissions
BOOL CReg::WriteStrVal(HKEY hHive, char lszKey[255], char lszValName[255], char lszValData[255])
{
        HKEY hKey;
        DWORD dwDisp = 0;
        LPDWORD lpdwDisp = &dwDisp;

        LONG iSuccess = RegCreateKeyEx( hHive, lszKey, 0L,NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey,lpdwDisp);

        if(iSuccess == ERROR_SUCCESS)
        {
	        RegSetValueEx(hKey, lszValName, 0, REG_SZ, (LPBYTE) lszValData, strlen(lszValData));
		RegCloseKey(hKey);

		return TRUE;
	}
	else
	{
		RegCloseKey(hKey);

		return FALSE;
	}
}


// PRE:	 HKEY containing hive, string containing key to access,
//		 string containing name of value to write,
//		 DWORD holding data to write as value
//
// POST: FALSE if operation failed, TRUE otherwise.
//       Reasons for failure include lack of permissions
BOOL CReg::WriteDwordVal(HKEY hHive, char lszKey[255], char lszValName[255], DWORD *dwVal)
{
       HKEY hKey;
       DWORD dwDisp = 0;
       LPDWORD lpdwDisp = &dwDisp;

       LONG iSuccess = RegCreateKeyEx(hHive, lszKey, 0L,NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL, &hKey,lpdwDisp);

       if(iSuccess == ERROR_SUCCESS)
       {
            RegSetValueEx (hKey, lszValName, 0L, REG_DWORD,(CONST BYTE*) &dwVal, sizeof(DWORD));
			RegCloseKey(hKey);

			return TRUE;
       }

	   else
	   {
			RegCloseKey(hKey);
			return FALSE;
	   }
}


// PRE:	 HKEY containing hive, string containing key to delete
//		 
// POST: FALSE if operation failed, TRUE otherwise. 
//       Reasons for faiulure include lack of permissions
BOOL CReg::DelKey(HKEY hHive, char lszKey[255])
{
	if(RegDeleteKey(hHive, lszKey)==ERROR_SUCCESS)
		return TRUE;
	else
		return FALSE;

}

BOOL CReg::DelVal(HKEY hive, char key[], char valueName[])
{
        HKEY hKey;
	LONG lRet, returnStatus;

	lRet = RegOpenKeyEx (hive, key, 0L, KEY_WRITE, &hKey);

        if(RegDeleteValue(hKey, "NetworkAddress")==ERROR_SUCCESS)
                return TRUE;
        else
                return FALSE;
}


// Experimental method meant to access registry remotely
// given that the client is in the same Windows domain as the
// server machine (the machine whose registry will be accessed
// remotely), and also given that administrative privileges are
// provided
// (This method is NOT finished yet)
BOOL CReg::RemoteTest()
{
	HKEY hKey;

	if(RegConnectRegistry("fortress", HKEY_LOCAL_MACHINE, &hKey)==ERROR_SUCCESS)
	{
		printf("handle value: %d\n", hKey);
		return TRUE;
	}
	else
	{
		printf("handle value: %d\n", hKey);
		return FALSE;
	}
}